use core::fmt;
use std::fmt::{Display, Formatter};

pub const XMR: Symbol = Symbol { name: "XMR" };
pub const BTC: Symbol = Symbol { name: "BTC" };
pub const ETH: Symbol = Symbol { name: "ETH" };
pub const LTC: Symbol = Symbol { name: "LTC" };
pub const USD: Symbol = Symbol { name: "USD" };
pub const GBP: Symbol = Symbol { name: "GBP" };
pub const EUR: Symbol = Symbol { name: "EUR" };


#[derive(Clone)]
pub struct Symbol<'a> {
    name: &'a str
}

impl<S> From<S> for Symbol where S:Into<String> {
    fn from(item: S) -> Self {
        Symbol { name: &item.into() }
    }
}


impl Symbol {
    pub fn new(name: &str) -> Self {
        Symbol { name }
    }

    pub fn name(&self) -> &str {
        &self.name
    }
}

impl Display for Symbol {
    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
        write!(f, "{}", self.name)
    }
}

#[derive(Clone)]
pub struct SymbolPair<'a> {
    quote: Symbol<'a>,
    base: Symbol<'a>,
}

impl SymbolPair {
    pub fn new(quote: Symbol, base: Symbol) -> Self {
        SymbolPair { quote, base }
    }

    pub fn trading_repr(&self) -> String {
        format!("t{}{}", self.quote, self.base)
    }

    pub fn funding_repr(&self) -> String {
        format!("f{}{}", self.quote, self.base)
    }

    pub fn quote(&self) -> &Symbol {
        &self.quote
    }
    pub fn base(&self) -> &Symbol {
        &self.base
    }
}

#[derive(Clone)]
enum WalletKind {
    Margin,
    Exchange,
    Funding,
}

#[derive(Clone)]
struct Balance<'a> {
    pair: SymbolPair<'a>,
    base_price: f64,
    base_amount: f64,
    quote_equivalent: f64,
    wallet: WalletKind,
}

impl Balance {
    pub fn new(pair: SymbolPair, base_price: f64, base_amount: f64, wallet: WalletKind) -> Self {
        Balance { pair, base_price, base_amount, quote_equivalent: base_amount * base_price, wallet }
    }

    pub fn pair(&self) -> &SymbolPair {
        &self.pair
    }
    pub fn base_price(&self) -> f64 {
        self.base_price
    }
    pub fn base_amount(&self) -> f64 {
        self.base_amount
    }
    pub fn quote_equivalent(&self) -> f64 {
        self.quote_equivalent
    }
    pub fn wallet(&self) -> &WalletKind {
        &self.wallet
    }
}

struct BalanceGroup<'a> {
    quote_equivalent: f64,
    balances: Vec<Balance<'a>>,
}

impl BalanceGroup {
    pub fn new() -> Self {
        BalanceGroup { balances: Vec::new(), quote_equivalent: 0f64 }
    }

    pub fn add_balance(&mut self, balance: &Balance) {
        self.balances.push(balance.clone());

        self.quote_equivalent += balance.quote_equivalent()
    }

    pub fn currency_names(&self) -> Vec<String> {
        self.balances.iter()
            .map(|x| x.pair().base().name().into())
            .collect()
    }

    pub fn balances(&self) -> &Vec<Balance> {
        &self.balances
    }
}



